package com.emato.cus.supervise.config.ds.primary;

import com.emato.cus.supervise.util.YmlUtils;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.PropertyResolver;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.sql.DataSource;

/**
 * 多数据源配置
 *
 * 数据源配置
 *
 * 指定mybatis-config.xml, 和要扫描的别名包
 *
 * @author Scott Chen
 * @date 2017/4/22
 */
@EnableTransactionManagement(order = 2)
@Configuration
public class PrimaryDataSourceConfig {

    @Autowired
    private PrimaryMybatisConfigProperties primaryMybatisConfigProperties;


    // -------------------- mybatis 属性配置 --------------------

    @Bean
    @ConfigurationProperties(prefix = "mybatis.primary")
    public PrimaryMybatisConfigProperties primaryMybatisConfigProperties(){
        return new PrimaryMybatisConfigProperties();
    }

    // -------------------- 主库 --------------------

    @Bean
    public DataSource primaryDataSource() {
        return primaryDataSourceProperties().initializeDataSourceBuilder().build();
    }

    @Bean
    @ConfigurationProperties(prefix = "spring.datasource.primary")
    public DataSourceProperties primaryDataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean
    public SqlSessionFactory primarySqlSessionFactory(DataSource primaryDataSource) throws Exception  {
        SqlSessionFactoryBean sessionFactoryBean = new SqlSessionFactoryBean();
        sessionFactoryBean.setDataSource(primaryDataSource);
        //Mybatis XML配置文件
        //配置mybatis-config.xml
        sessionFactoryBean.setConfigLocation(new PathMatchingResourcePatternResolver().getResource(primaryMybatisConfigProperties.getConfigLocation()));
        //配置mapper.xml
        sessionFactoryBean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources(primaryMybatisConfigProperties.getMapperLocationsConfig()));
        //配置扫描type aliases的包路径
        sessionFactoryBean.setTypeAliasesPackage(primaryMybatisConfigProperties.getTypeAliasesPackage());
        return sessionFactoryBean.getObject();
    }

    @Bean
    public DataSourceTransactionManager primaryTransactionManager(DataSource primaryDataSource) {
        return new DataSourceTransactionManager(primaryDataSource);
    }


    @Bean
    public SqlSessionTemplate primarySqlSessionTemplate(SqlSessionFactory primarySqlSessionFactory) {
        return new SqlSessionTemplate(primarySqlSessionFactory);
    }


    /*private String configLocationByYml() {
        String path = MYBATIS_MAPPER_CONFIG_YML_PATH;
        String key = "mybatis.primary.config-location";
        return YmlUtils.instance().getValue(key, path);
    }

    private String mapperLocationsByYml() {
        String path = MYBATIS_MAPPER_CONFIG_YML_PATH;
        String key = "mybatis.primary.mapper-locations-config";
        return YmlUtils.instance().getValue(key, path);
    }

    private String typeAliasesByYml() {
        String path = MYBATIS_MAPPER_CONFIG_YML_PATH;
        String key = "mybatis.primary.type-aliases-package";
        return YmlUtils.instance().getValue(key, path);
    }*/

}
